% Matlab script built by Pavlo Bazilinskyy and Joost de Winter
% <pavlo.bazilinskyy@gmail.com>
%
% Maintained version of the code that also produces additional figures is 
% available at https://github.com/bazilinskyy/text-crowdsourced
clear all;close all;clc; %#ok<*CLALL>

%% ************************************************************************
%% Constants
%% **********[**************************************************************
N_STIMULI = 227;  % number of stimuli
N_PERSON = 80;    % number of stimuli per person
N_SUB = 20;       % number of top and bottom stimuli to show in
                  % barplots
STEP_COLOUR = 5;  % step for traversing over colourmap
COLOUR_SAME_EHMI = true;  % flag for colouring eHMI in ES/EN on figures
                          % with all eHMI

%% ************************************************************************
%% Load config
%% **************************there*****************************************
config = jsondecode(fileread('config'));

%% ************************************************************************
%% Process data
%% ************************************************************************
% indices to traverse in appen data
appen_indices = [19,... % 1. Instructions understood
                 39,... % 2. Gender
                 38,... % 3. Age
                 17,... % 4. Age of obtaining driver's license
                 42,... % 5. Primary mode of transportation
                 34,... % 6. How many times in past 12 months did you drive a vehicle
                 16,... % 7. Mileage
                 20,... % 8. Number of accidents
                 12,...  % Country
                 21,... % 9. DBQ1
                 22,... % 10. DBQ2
                 23,... % 11. DBQ3
                 24,... % 12. DBQ4
                 25,... % 13. DBQ5
                 26,... % 14. DBQ6
                 27,... % 15. DBQ7
                 40,... % 16. Knowledge of English
                 41,... % 17. Knowledge of Spanish
                 18,... % 18. English 1
                 37,... % 19. English 2
                 28,... % 20. English 3
                 29,... % 21. English 4
                 36,... % 22. English 5
                 7,...  % 23. Start
                 5,...  % 24. End
                 11,...  % 321. Worker id
                 2,...  % worker_code
                 15];   % IP
% process data using external function
[X, Country] = process_experiment(config.file_appen, ...
                                  appen_indices, ...
                                  config.file_heroku, ...
                                  N_STIMULI);

%% Read mapping of eHMIs
mapping = readtable(config.mapping);

%% Prepare data for output
RT = X(:, 27:106);       % time used to press the key
RP = X(:, 107:186);      % response from the slider
imageid = X(:,187:266);  % image ids as shown
lang_br = X(:,267);      % browser's language (1=Spanish, 0=not Spanish)
eHMI_text=mapping(:,2);  % labels with eHMIs
% Order based on image number
[RPo,RTo]=deal(NaN(size(RP,1),N_STIMULI));
for i=1:size(RP,1) % loop over pp
    RPo(i,imageid(i,:)+1)=RP(i,:);
    RTo(i,imageid(i,:)+1)=RT(i,:);
end
% Median willingness to cross
[RPoMed,RPoMedVE,RPoMedUS,RToMedVE,RToMedUS]=deal(NaN(N_STIMULI,1));
for i=1:size(RPo,2)
    % equal medians sorted on SD
    RPoMed(i)=nanmedian(RPo(:,i))-nanstd(RPo(:,i))/10^6;
    % mean for participants from VEN
    RPoMedVE(i)=nanmedian(RPo(contains(Country,'VE'),i));
    % mean for participants from USA
    RPoMedUS(i)=nanmedian(RPo(contains(Country,'US'),i));
    % median RT participants from VEN
    RToMedVE(i)=nanmedian(RTo(contains(Country,'VE'),i));
    % median RT participants from USA
    RToMedUS(i)=nanmedian(RTo(contains(Country,'US'),i));
end
[RPoMedSorted,b]=sort(RPoMed);
eHMI_text_MedSorted=char(eHMI_text{b,:});
% Mean willingness to cross
[RPoMean,RPoMeanVE,RPoMeanUS,RToMeanVE,RToMeanUS,o,NN]=deal(NaN(N_STIMULI,1));
for i=1:size(RPo,2)
    % equal mean sorted on SD
    RPoMean(i)=nanmean(RPo(:,i))-nanstd(RPo(:,i))/10^6;
    % mean for participants from VEN
    RPoMeanVE(i)=nanmean(RPo(contains(Country, 'VE'),i));
    % mean for participants from USA
    RPoMeanUS(i)=nanmean(RPo(contains(Country, 'US'),i));
    % mean RT participants from VEN
    RToMeanVE(i)=nanmean(RTo(contains(Country, 'VE'),i));
    % mean RT participants from USA  
    RToMeanUS(i)=nanmean(RTo(contains(Country, 'US'),i)); 
end
[RPoMeanSorted,b]=sort(RPoMean);
eHMI_text_MeanSorted=char(eHMI_text{b,:});
% SD willingness to cross
RPoSTD=deal(NaN(N_STIMULI,1));
for i=1:size(RPo,2)
    RPoSTD(i)=nanstd(RPo(:,i));
end
[RPoSTDSorted, bs]=sort(RPoSTD);
eHMI_text_STDSorted=char(eHMI_text{bs,:});
% Types of eHMIs
ego=find(mapping{:,7}==1 & mapping{:,8}==0);
allo=find(mapping{:,7}==0 & mapping{:,8}==1);
other=find(~ismember(1:227, union(ego,allo)));

%% ************************************************************************
%% OUTPUT
%% ************************************************************************
set(0, 'DefaultFigurePosition', [5 60  1920/2 1080/2]);
opengl hardware

%% Figure 2. Multi-column barplot for crossing percentage
[RPo_mean_sorted,RPO_mean_sorted_o]=sort(nanmean(RPo),'ascend');
% prepare data
cd=NaN(size(RPo,2),3);
cd(ego,:)=repmat([0 .9 0], length(ego), 1);
cd(allo,:)=repmat([0.8 0.8 0.8], length(allo), 1);
cd(other,:)=repmat([1 .5 0], length(other), 1);
% 1st column
figure
subplot(1,4,1)
b=barh(RPo_mean_sorted(1:114),'facecolor','flat');
b.CData=cd(RPO_mean_sorted_o(1:114),:);
for i=1:114
    text(1,i,eHMI_text{RPO_mean_sorted_o(i),:},'color','k','fontsize',6)
end
set(gca,'xlim',[0 85], ...
    'pos',[0.01 0.045 0.22 0.95], ...
    'yticklabel', {}, ...
    'ytick',[], ...
    'ticklength', [0.005 0], ...
    'ydir', 'reverse')
xlabel('Mean willingness to cross (%)')
% 2nd column
subplot(1,4,2)
b=barh(RPo_mean_sorted(115:227),'facecolor',[.8 .8 .8],'facecolor','flat');
b.CData=cd(RPO_mean_sorted_o(115:227),:);
for i=115:227
    text(1,i-114,eHMI_text{RPO_mean_sorted_o(i),:},'color','k','fontsize',6)
end
set(gca,'xlim',[0 85], ...
    'pos',[0.24 0.045 0.22 0.95], ...
    'yticklabel', {}, ...
    'ytick',[], ...
    'ticklength', [0.005 0], ...
    'ydir', 'reverse')
xlabel('Mean willingness to cross (%)')
% maximise and export as eps and jpg (for readme)
if config.save_figures
    export_figure(gcf, [config.path_output filesep 'figures' ...
                        filesep 'mean-cross-multiple-columns'], 'epsc')
    export_figure(gcf, [config.path_figures ...
                        filesep 'mean-cross-multiple-columns'], 'jpg')
    % crop figure
    img = imread([config.path_figures filesep ...
                  'mean-cross-multiple-columns.jpg']);
    [rows, columns, numberOfColorChannels] = size(img);
    img2 = imcrop(img, [0 0 columns/2 - columns/30 rows]);
    imwrite(img2, ...
           [config.path_figures filesep 'mean-cross-multiple-columns.jpg'], ...
           'jpg');
end

%% Figure 3. Text scatter plot of English-text eHMIs. Median response time
% and median response time
figure;hold on;box on
cd=NaN(180,3);
cd(ego,:)=repmat([0 0.8 0], length(ego), 1);
cd(allo,:)=repmat([0 0 0], length(allo), 1);
cd(other,:)=repmat([1 0 0], length(other), 1);
cd=cd(1:180,:);
plot(-10,-10,'o','markerfacecolor',cd(ego(1),:),'markersize',10)
plot(-10,-10,'o','markerfacecolor',cd(allo(1),:),'markersize',10)
plot(-10,-10,'o','markerfacecolor',cd(other(1),:),'markersize',10)
h = textscatter([nanmedian(RTo(:,1:180))' nanstd(RPo(:,1:180))'], ...
                table2cell(eHMI_text(1:180, :)), ...
                'markersize', 22, ...
                'colordata', cd, ...
                'TextDensityPercentage', 67, ...
                'maxtextlength', 100, ...
                'fontsize', 14);
xlabel('Median response time (ms)');
ylabel('\it{SD}\rm willingness to cross (%)');
set(gca, 'Fontsize', 20, ...
    'LooseInset', [0.01 0.01 0.01 0.01], ...
    'xlim', [3600 7100], ...
    'ylim', [20 35], ...
    'ticklength', [0.005 0.005])
legend('Egocentric', 'Allocentric', 'Egocentric and allocentric', ...
       'location', 'southeast')
% maximise and export as eps and jpg (for readme)
if config.save_figures
    export_figure(gcf, [config.path_output filesep 'figures' ...
                        filesep 'scatter-text-en-median'], 'epsc')
    export_figure(gcf, [config.path_figures ...
                        filesep 'scatter-text-en-median'], 'jpg')
end

%% Figure 4. Scatter plot for Spanish and corresponding English eHMI texts
% assign colours to pairs of EN and ES eHMIs
Ewi=NaN(47,1);
for i=1:47 % loop over 47 Spanish eHMI texts
    % find the index of the corresponding English eHMI text
    Ewi(i)=find(strcmp(mapping{:,2},mapping{180+i,10}));
end
figure;
hold on;
grid on;
box on
for i=1:47
    scatter1 = scatter(nanmean(RPo(lang_br==1,Ewi(i))), ...
                       nanmean(RPo(lang_br==1,i+180)), ...
                       250, ...
                       'markerfacecolor', [255, 204, 0]/255, ...
                       'markeredgecolor', 'none');
    scatter2 = scatter(nanmean(RPo(lang_br==0,Ewi(i))), ...
                       nanmean(RPo(lang_br==0,i+180)), ...
                       250, ...
                       'markerfacecolor', [179, 25, 66]/255, ...
                       'markeredgecolor', 'none');
    plot([nanmean(RPo(lang_br==1,Ewi(i))) nanmean(RPo(lang_br==0,Ewi(i)))], ...
         [nanmean(RPo(lang_br==1,i+180)) nanmean(RPo(lang_br==0,i+180))], ...
         'k--')
    scatter1.MarkerFaceAlpha = 0.8;
    scatter2.MarkerFaceAlpha = 0.8;
end
    plot([0 100],[0 100],'b--')
legend('Participants with browser language Spanish', ...
       'Participants with browser language English', ...
       'location','southeast')
xlabel('Mean willingness to cross - eHMI in English (%)');
ylabel('Mean willingness to cross - eHMI in Spanish (%)');
h=findobj('FontName', 'Helvetica');
set(h,'FontSize', 20, 'Fontname', 'Arial')
set(gca, 'LooseInset', [0.01 0.01 0.01 0.01], ...
    'xlim', [0 100], ...
    'ylim', [0 100], ...
    'pos', [0.25 0.08 0.5 0.9])
if config.save_figures
% maximise and export as eps and jpg (for readme)
    export_figure(gcf, [config.path_output filesep 'figures' ...
                        filesep 'ehmis-en-es'], 'epsc')
    export_figure(gcf, [config.path_figures ...
                        filesep 'ehmis-en-es'], 'jpg')
end

%% Display correlation matrices at the level of stimuli
XCM=[2*abs(nanmean(RPo(:,1:180)-50))' nanstd(RPo(:,1:180))' nanmedian(RTo(:,1:180))' mapping{1:180,[3 7 8]}];
disp([datestr(now, 'HH:MM:SS.FFF') ' - Correlation matrix all participants, Enligh-text eHMIs'])
disp(round(corr(XCM),2))

%% Table. Regression model statistics
y=XCM(:,3);
x=XCM(:,4:5);
stu=regstats(y, x);
y=zscore(y);x=zscore(x);
st=regstats(y, x);
disp([round([stu.beta st.beta st.tstat.t ], 3), ...
      round(1000*st.tstat.pval(1:end))/1000])
disp([datestr(now, 'HH:MM:SS.FFF') ' - Number of trials in the ' ...
      'regression analysis = ' num2str(length(y))])
disp([st.fstat.dfr st.fstat.dfe st.fstat.f st.fstat.pval ...
      corr(st.yhat,y) st.rsquare])

%% Information on browser language
% participants with browser language English
disp([datestr(now, 'HH:MM:SS.FFF') ' - Participants with browser language English:'])
disp(nanmean(X(lang_br==0,[16 17 23])))
% participants with browser language Spanish
disp([datestr(now, 'HH:MM:SS.FFF') ' - Participants with browser language Spanish:'])
disp(nanmean(X(lang_br==1,[16 17 23])))

%% ************************************************************************
%% Export of overview of eHMIs to csv
%% ************************************************************************
t = table(mapping{:,2}, ...
          RPoMed, ...
          RPoMean, ...
          RPoSTD, ...
          'VariableNames', {'eHMI' 'med' 'mean' 'std'});
% sort by median of willingness to cross
t_med = sortrows(t, 2, 'descend');
% sort by mean of willingness to cross
t_mean = sortrows(t, 3, 'descend');
% sort by std of willingness to cross
t_std = sortrows(t, 4, 'descend');
% export to csv
writetable(t, [config.path_output filesep 'ehmis.csv']);
writetable(t_med, [config.path_output filesep 'ehmis_med.csv']);
writetable(t_mean, [config.path_output filesep 'ehmis_mean.csv']);
writetable(t_std, [config.path_output filesep 'ehmis_std.csv']);
